@@ -6,18 +6,15 @@ package Business::Tax::VAT::Validation;
 =cut
 
  ############################################################################
-# Original author:                                                           #
 # IT Development software                                                    #
-# European VAT number validator Version 1.0.2                                #
-# Created 06/08/2003                                                         #
-#                                                                            #
-# Maintainership kindly handed over to David Precious (BIGPRESH) in 2015     #
+# European VAT number validator Version 1.01                                 #
+# Copyright 2003 Nauwelaerts B  bpgn@cpan.org                                #
+# Created 06/08/2003            Last Modified 25/03/2012                     #
  ############################################################################
 # COPYRIGHT NOTICE                                                           #
 # Copyright 2003 Bernard Nauwelaerts  All Rights Reserved.                   #
-# Copyright 2015 David Precious       All Rights Reserved.                   #
 #                                                                            #
-# THIS SOFTWARE IS RELEASED UNDER THE GNU Public Licence version 3           #
+# THIS SOFTWARE IS RELEASED UNDER THE GNU Public Licence                     #
 # Please see COPYING for details                                             #
 #                                                                            #
 # DISCLAIMER                                                                 #
@@ -25,19 +22,79 @@ package Business::Tax::VAT::Validation;
 #  WITHOUT ANY WARRANTY, without even the implied warranty of                #
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      #
 #                                                                            #
-############################################################################
+ ############################################################################
+# Revision history (dd/mm/yyyy) :                                            #
+#                                                                            #
+# 1.01   28/11/2012; Fix POD UTF8 Issue; Thanks to Graham Knop               #
+# 1.00   25/03/2012; This module now uses the VIES SOAP interface.           #
+#                    Add get_informations method                             #
+# 0.24   06/03/2012; Fix traderName field required for EL and ES MS          #
+#                    Update POST request fields                              #
+# 0.23   29/02/2012; Fix regexp in _is_res_ok with multiline regexp          #
+#                    (Thanks to Bart Heupers)                                #
+# 0.22   04/10/2011; Typo fix in POD error message #2                        #
+#                    (Thanks to Martin H. Sluka)                             #
+#                    Minor POD fixes (BPGN)                                  #
+#                    Fix subs args bug (performance fix) (BPGN)              #
+# 0.21   04/08/2009; New error status 19: EU database too busy               #
+#                    (Martin H. Sluka)                                       #
+# 0.20   18/08/2008; VIES HTML changes: now allowing multiples spaces        #
+#                    (Thanks to Simon Williams, Benoît Galy & Raluca Boboia) #
+# 0.19   29/04/2008; HU regexp: missing digit "9" added                      #
+#                    (Thanks to Simon Williams)                              #
+# 0.18   03/01/2008; BE regexp: from transitional 9-digit & 10-digit format  #
+#                    to 10-digit new format                                  #
+# 0.16   13/07/2007; Allowing spaces in regexps                              #
+# 0.15   06/07/2007; Added missing "keys" during $self->{members}            #
+#                    constuction                                             #
+#                    (Thanks to Dave O.)                                     #
+#                    Corrected returned 0-1 error codes inversion            #
+#                    Some POD improvements                                   #
+# 0.14   05/07/2007; VIES interface changed $baseurl, and POST params were   #
+#                    changed to lowercase. Added support for Bulgaria and    #
+#                    Romania (thanks to Kaloyan Iliev)                       #
+#                    New get_last_error_code method                          #
+#                    Updated regexps according to the actual VIES FAQ        #
+#                    Some slight documentation improvements                  #
+#                    Improved tests: each regexp is tested accordind to FAQ  #
+# 0.13   16/01/2007; VIES interface changed "not found" layout               #
+#                    (Thanks to Tom Kirkpatrick for this update)             #
+# 0.12   10/11/2006; YAML Compliance                                         #
+# 0.11   10/11/2006; Minor bug allowing one forbidden character              #
+#                    corrected in Belgian regexp                             #
+#                     (Thanks to Andy Wardley for this report)               #
+#                    + added regular_expressions property                    #
+#                      for external testing purposes                         #
+# 0.10   20/07/2006; Adding Test::Pod to test suite                          #
+# 0.09   20/06/2006; local_check method allows you to test VAT numbers       #
+#                    without asking the EU database. Based on regexps.       #
+# 0.08   20/06/2006; 9 and 10 digits transitional regexp for Belgium         #
+#                    From 31/12/2007, only 10 digits will be valid           #
+# 0.07   25/05/2006; Now we use "request" method not "simple request"        #
+#                    in order to follow potential redirects                  #
+# 0.06   25/05/2006; Changed $baseurl                                        #
+#                    (Thanks to Torsten Mueller for this update)             #
+# 0.05   19/01/2006; Adding support for proxy settings                       #
+#                    (Thanks to Tom Kirkpatrick for this update)             #
+# 0.04   01/11/2004; Adding support for error "Member Service Unavailable"   #
+# 0.03   01/11/2004; Adding 10 new members.                                  #
+#                    (Thanks to Robert Alloway for this update)              #
+# 0.02   29/09/2003; Fix alphanumeric VAT numbers rejection                  #
+#                    (Thanks to Robert Alloway for the regexps)              #
+# 0.01   06/08/2003; Initial release                                         #
+#                                                                            #
+ ############################################################################
 use strict;
-use warnings;
 
-our $VERSION = '1.13_01';
-
-use HTTP::Request::Common qw(POST);
-use LWP::UserAgent;
-use JSON qw/ decode_json /;
+BEGIN {
+    $Business::Tax::VAT::Validation::VERSION = '1.01';
+    use HTTP::Request::Common qw(POST);
+    use LWP::UserAgent;
+}
 
 =head1 NAME
 
-Business::Tax::VAT::Validation - Validate EU VAT numbers against VIES
+Business::Tax::VAT::Validation - A class for european VAT numbers validation.
 
 =head1 SYNOPSIS
 
@@ -54,16 +111,10 @@ Business::Tax::VAT::Validation - Validate EU VAT numbers against VIES
   
 =head1 DESCRIPTION
 
-This class provides an easy API to check European VAT numbers' syntax,
-and if they has been registered by the competent authorities.
+This class provides an easy api to check european VAT numbers' syntax, and if they has been registered by the competent authorities.
 
-It asks the EU database (VIES) for this, using its SOAP API.  Basic checks that
-the supplied VAT number fit the expected format for the specified EU member
-state are performed first, to avoid unnecessarily sending queries to VIES for
-input that could never be valid.
+It asks the EU database (VIES) for this, using its SOAP interface methods. 
 
-It also supports looking up VAT codes from the United Kingdom by using the
-REST API provided by their HMRC.
 
 =head1 CONSTRUCTOR
 
@@ -85,11 +136,9 @@ REST API provided by their HMRC.
 sub new {
     my ( $class, %arg ) = @_;
     my $self = {
-        baseurl      => $arg{baseurl} || 'https://ec.europa.eu/taxation_customs/vies/services/checkVatService',
-        hmrc_baseurl => $arg{hmrc_baseurl} || 'https://api.service.hmrc.gov.uk/organisations/vat/check-vat-number/lookup/',
+        baseurl      => $arg{baseurl} || 'http://ec.europa.eu/taxation_customs/vies/services/checkVatService',
         error        => '',
         error_code   => 0,
-        response     => '',
         re           => {
             ### t/01_localcheck.t tests if these regexps accepts all regular VAT numbers, according to VIES FAQ
             AT => 'U[0-9]{8}',
@@ -105,9 +154,8 @@ sub new {
             FI => '[0-9]{8}',
             FR => '[A-Za-z0-9]{2} ?[0-9]{9}',
             GB => '([0-9]{3} ?[0-9]{4} ?[0-9]{2}|[0-9]{3} ?[0-9]{4} ?[0-9]{2} ?[0-9]{3}|GD[0-9]{3}|HA[0-9]{3})',
-            HR => '[0-9]{11}',
             HU => '[0-9]{8}',
-            IE => '[0-9][A-Za-z0-9\+\*][0-9]{5}[A-Za-z]{1,2}',
+            IE => '[0-9][A-Za-z0-9\+\*][0-9]{5}[A-Za-z]',
             IT => '[0-9]{11}',
             LT => '([0-9]{9}|[0-9]{12})',
             LU => '[0-9]{8}',
@@ -120,10 +168,9 @@ sub new {
             SE => '[0-9]{12}',
             SI => '[0-9]{8}',
             SK => '[0-9]{10}',
-            XI => '([0-9]{3} ?[0-9]{4} ?[0-9]{2}|[0-9]{3} ?[0-9]{4} ?[0-9]{2} ?[0-9]{3}|GD[0-9]{3}|HA[0-9]{3})',
         },
         proxy        => $arg{-proxy},
-        information => {}
+        informations => {}
     };
     $self = bless $self, $class;
     $self->{members} = join( '|', keys %{ $self->{re} } );
@@ -136,23 +183,7 @@ sub new {
 
 =over 4
 
-=item B<member_states> Returns all supported country codes.
-
-These are ISO 3166-1 alpha-2 country codes with two exceptions. This module
-supports VAT codes from all current European Union member states and The United
-Kingdom of Great Britain and Northern Ireland.
-
-=over 4
-
-=item C<EL> Greece
-
-Must be used in place of Greece's proper code.
-
-=item C<XI> Northern Ireland
-
-May be used rather than C<GB> for checking a Northern Irish company.
-
-=back
+=item B<member_states> Returns all member states 2-digit codes as array
 
     @ms=$hvatn->member_states;
     
@@ -200,22 +231,33 @@ or specify the VAT and MSC (vatNumber and countryCode) individually.
 Valid MS values are :
 
  AT, BE, BG, CY, CZ, DE, DK, EE, EL, ES,
- FI, FR, GB, HR, HU, IE, IT, LU, LT, LV,
- MT, NL, PL, PT, RO, SE, SI, SK, XI
+ FI, FR, GB, HU, IE, IT, LU, LT, LV, MT,
+ NL, PL, PT, RO, SE, SI, SK
 
 =cut
 
 sub check {
     my ($self, $vatNumber, $countryCode, @other) = @_;    # @other is here for backward compatibility purposes
-    $self->{information} = {};
     return $self->_set_error('You must provide a VAT number') unless $vatNumber;
     $countryCode ||= '';
     ( $vatNumber, $countryCode ) = $self->_format_vatn( $vatNumber, $countryCode );
     if ($vatNumber) {
-        if ($countryCode eq 'GB') {
-            return $self->_check_hmrc($vatNumber, $countryCode);
+        my $ua = LWP::UserAgent->new;
+        if ( ref $self->{proxy} eq 'ARRAY' ) {
+            $ua->proxy( @{ $self->{proxy} } );
+        } else {
+            $ua->env_proxy;
         }
-        return $self->_check_vies($vatNumber, $countryCode);
+        $ua->agent( 'Business::Tax::VAT::Validation/'. $Business::Tax::VAT::Validation::VERSION );
+        
+        my $request = HTTP::Request->new(POST => $self->{baseurl});
+        $request->header(SOAPAction => 'http://www.w3.org/2003/05/soap-envelope');
+        $request->content(_in_soap_envelope($vatNumber, $countryCode));
+        $request->content_type("Content-Type: application/soap+xml; charset=utf-8");
+        
+        my $response = $ua->request($request);
+        
+        return $countryCode . '-' . $vatNumber if $self->_is_res_ok( $response->code, $response->decoded_content );
     }
     0;
 }
@@ -230,7 +272,6 @@ sub check {
 
 sub local_check {
     my ( $self, $vatn, $mscc, @other ) = @_;    # @other is here for backward compatibility purposes
-    $self->{information} = {};
     return $self->_set_error('You must provide a VAT number') unless $vatn;
     $mscc ||= '';
     ( $vatn, $mscc ) = $self->_format_vatn( $vatn, $mscc );
@@ -242,28 +283,19 @@ sub local_check {
     }
 }
 
-=item B<information> - Returns information related to the last checked VAT number
-
-    # Get all available information as a hashref:
-    my $info = $hvatn->information();
-
-    # Get a particular key:
-    my $address = $hvatn->information('address');
-
-Which information is offered depends on the checker used - for UK VAT numbers,
-checked via the HMRC API, C<address> is the only key which will be set.
-
-For EU VAT numbers checked via VIES, you can expect C<name> and C<address>.
-This hashref will be reset every time you call check() or local_check()
+=item B<informations> - Returns informations related to the last validated VAT number
+    
+    %infos=$hvatn->informations();
+    
 
 =cut
 
-sub information {
+sub informations {
     my ( $self, $key, @other ) = @_; 
     if ($key) {
-        return $self->{information}{$key} 
+        return $self->{informations}{$key} 
     } else {
-        return ($self->{information})    
+        return ($self->{informations})    
     }
 }
 
@@ -331,76 +363,7 @@ sub get_last_error_code {
     $_[0]->{error_code};
 }
 
-=item B<get_last_response> - Returns the full last response
-
-=cut
-
-sub get_last_response {
-    $_[0]->{response};
-}
-
 ### PRIVATE FUNCTIONS ==========================================================
-sub _get_ua {
-    my ($self) = @_;
-    my $ua = LWP::UserAgent->new;
-    if ( ref $self->{proxy} eq 'ARRAY' ) {
-        $ua->proxy( @{ $self->{proxy} } );
-    } else {
-        $ua->env_proxy;
-    }
-    $ua->agent( 'Business::Tax::VAT::Validation/'. $Business::Tax::VAT::Validation::VERSION );
-    return $ua;
-}
-
-sub _check_vies {
-  my ($self, $vatNumber, $countryCode) = @_;
-  my $ua = $self->_get_ua();
-  my $request = HTTP::Request->new(POST => $self->{baseurl});
-  $request->header(SOAPAction => 'http://www.w3.org/2003/05/soap-envelope');
-  $request->content(_in_soap_envelope($vatNumber, $countryCode));
-  $request->content_type("Content-Type: application/soap+xml; charset=utf-8");
-
-  my $response = $ua->request($request);
-
-  return $countryCode . '-' . $vatNumber if $self->_is_res_ok( $response->code, $response->decoded_content );
-}
-
-sub _check_hmrc {
-    my ($self, $vatNumber, $countryCode) = @_;
-    my $ua = $self->_get_ua();
-
-    my $request = HTTP::Request->new(GET => $self->{hmrc_baseurl}.$vatNumber);
-    $request->header(Accept => 'application/vnd.hmrc.1.0+json');
-    my $response = $ua->request($request);
-
-    $self->{res} = $response->decoded_content;
-    if ($response->code == 200) {
-        my $data = decode_json($self->{res});
-        $self->{information}->{name} = $data->{target}->{name};
-        my $line = 1;
-        my $address = "";
-        while (defined $data->{target}->{address}->{"line$line"}) {
-            $address .= $data->{target}->{address}->{"line$line"}."\n";
-            $line++;
-        }
-        $address .= $data->{target}->{address}->{postcode};
-        $address .= "\n".$data->{target}->{address}->{countryCode};
-        $self->{information}->{address} = $address;
-        $self->_set_error( -1, 'Valid VAT Number');
-    }
-    elsif ($response->code == 404) {
-        return $self->_set_error( 2, 'Invalid VAT Number ('.$vatNumber.')');
-    }
-    elsif ($response->code == 400) {
-        return $self->_set_error( 3, 'VAT number badly formed ('.$vatNumber.')');
-    }
-    else {
-        return $self->_set_error( 500, 'Could not contact HMRC: '.$response->status_line);
-    }
-
-    return $countryCode . '-' . $vatNumber;
-}
-
 sub _format_vatn {
     my ( $self, $vatn, $mscc ) = @_;
     my $null = '';
@@ -438,18 +401,17 @@ sub _in_soap_envelope {
 
 sub _is_res_ok {
     my ( $self, $code, $res ) = @_;
-    $self->{information}={};
+    $self->{informations}={};
     $res=~s/[\r\n]/ /g;
-    $self->{response} = $res;
     if ($code == 200) {
         if ($res=~m/<valid> *(.*?) *<\/valid>/) {
             my $v = $1;
             if ($v eq 'true' || $v eq '1') {
                 if ($res=~m/<name> *(.*?) *<\/name>/) {
-                    $self->{information}{name} = $1
+                    $self->{informations}{name} = $1
                 }
                 if ($res=~m/<address> *(.*?) *<\/address>/) {
-                    $self->{information}{address} = $1
+                    $self->{informations}{address} = $1
                 }
                 $self->_set_error( -1, 'Valid VAT Number');
                 return 1;
@@ -493,10 +455,8 @@ sub _set_error {
 
 LWP::UserAgent
 
-L<http://ec.europa.eu/taxation_customs/vies/faqvies.do> for the FAQs related to the VIES service.
+I<http://ec.europa.eu/taxation_customs/vies/faqvies.do> for the FAQs related to the VIES service.
 
-L<https://developer.service.hmrc.gov.uk/api-documentation/docs/api/service/vat-registered-companies-api/1.0>
-for details of the service provided by the UK's HMRC.
 
 =head1 FEEDBACK
 
@@ -505,9 +465,7 @@ If you find this module useful, or have any comments, suggestions or improvement
 
 =head1 AUTHOR
 
-Original author: Bernard Nauwelaerts <bpgn@cpan.org>
-
-Maintainership since 2015: David Precious (BIGPRESH) <davidp@preshweb.co.uk>
+Bernard Nauwelaerts <bpgn@cpan.org>
 
 
 =head1 CREDITS
@@ -516,9 +474,6 @@ Many thanks to the following people, actively involved in the development of thi
 
 =over 4
 
-=item *
-Gregor Herrmann, Debian.
-
 =item *
 Graham Knop.
 
@@ -529,13 +484,7 @@ Bart Heupers, Netherlands.
 Martin H. Sluka, noris network AG, Germany.
 
 =item *
-Simon Williams, UK2 Limited, United Kingdom
-
-=item *
-Benoît Galy, Greenacres, France
-
-=item *
-Raluca Boboia, Evozon, Romania
+Simon Williams, UK2 Limited, United Kingdom & Benoît Galy, Greenacres, France & Raluca Boboia, Evozon, Romania
 
 =item *
 Dave O., POBox, U.S.A.
@@ -555,20 +504,16 @@ Robert Alloway, Service Centre, United Kingdom.
 =item *
 Torsten Mueller, Archesoft, Germany
 
-item *
-
-Dave Lambley (davel), United Kingdom
-
 =back
 
 =head1 LICENSE
 
-GPL3. Enjoy! See COPYING for further information on the GPL.
+GPL. Enjoy! See COPYING for further informations on the GPL.
 
 
 =head1 DISCLAIMER
 
-See L<http://ec.europa.eu/taxation_customs/vies/viesdisc.do> to known the limitations of the EU validation service.
+See I<http://ec.europa.eu/taxation_customs/vies/viesdisc.do> to known the limitations of the EU validation service.
 
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
   without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
